# Jorgensen, Chandrasekhar, Madura, Impey, Klein, J Chem Phys, 79, 926 (1983)

citation = "Jorgensen, Chandrasekhar, Madura, Impey, Klein, J Chem Phys, 79, 926 (1983)"

molecule = "HOH"

params = {"Z_H": 0.520,
          "Z_O": -1.040,
          "r0": 0.9572,
          "theta": 104.52,
          "OM": 0.15,
          "epsilon": 0.1550,
          "sigma": 3.1536,
          "r4s": 8.5}
